@@ -18,6 +18,8 @@
 from contextlib import suppress
 from shutil import copyfileobj
 from pathlib import Path
+from datetime import datetime, timezone
+from io import BytesIO
 import sys
 import os
 import shutil
@@ -31,7 +33,7 @@ from ruffus import formatter, regex, Pipeline, suffix
 
 from .hocrtransform import HocrTransform
 from .pdfinfo import PdfInfo, Encoding, Colorspace
-from .pdfa import generate_pdfa_ps
+from .pdfa import generate_pdfa_ps, encode_pdf_date
 from .helpers import re_symlink, is_iterable_notstr, page_number
 from .exec import ghostscript, tesseract, qpdf
 from .lib import fitz
@@ -143,6 +145,13 @@ def triage_image_file(input_file, output_file, log, options):
                 "image was scanned and specify it using --image-dpi.")
             raise DpiError()
 
+        if im.mode in ('RGBA', 'LA'):
+            log.error(
+                "The input image has an alpha channel. Remove the alpha "
+                "channel first."
+            )
+            raise UnsupportedImageFormatError()
+
         if 'iccprofile' not in im.info:
             if im.mode == 'RGB':
                 log.info('Input image has no ICC profile, assuming sRGB')
@@ -171,7 +180,7 @@ def triage_image_file(input_file, output_file, log, options):
 
 def _pdf_guess_version(input_file, search_window=1024):
     """Try to find version signature at start of file.
-    
+
     Not robust enough to deal with appended files.
 
     Returns empty string if not found, indicating file is probably not PDF.
@@ -221,7 +230,7 @@ def repair_and_parse_pdf(
 
     pdfinfo = PdfInfo(output_file)
 
-    if pdfinfo.has_userunit and options.output_type == 'pdfa':
+    if pdfinfo.has_userunit and options.output_type.startswith('pdfa'):
         log.error(
             "This input file uses a PDF feature that is not supported "
             "by Ghostscript, so you cannot use --output-type=pdfa for this "
@@ -231,6 +240,12 @@ def repair_and_parse_pdf(
         )
         raise InputFileError()
 
+    if len(pdfinfo.pages) > 2000 and sys.version_info[0:2] <= (3, 5):
+        log.warning(
+            "Performance regressions are known occur with Python 3.5 for "
+            "high page count files.  Python 3.6 or newer is recommended."
+        )
+
     context.set_pdfinfo(pdfinfo)
     log.debug(pdfinfo)
 
@@ -291,7 +306,7 @@ def is_ocr_required(pageinfo, log, options):
         # We found a page with no images and no text. That means it may
         # have vector art that the user wants to OCR. If we determined
         # lossless reconstruction is not possible then we have to rasterize
-        # the image. So if OCR is being forced, take that to mean YES, go 
+        # the image. So if OCR is being forced, take that to mean YES, go
         # ahead and rasterize. If not forced, then pretend there's no text
         # on the page at all so we don't lose anything.
         # This could be made smarter by explicitly searching for vector art.
@@ -489,17 +504,23 @@ def rasterize_with_ghostscript(
     options = context.get_options()
     pageinfo = get_pageinfo(input_file, context)
 
-    device = 'png16m'  # 24-bit
-    if pageinfo.images:
-        if all(image.comp == 1 for image in pageinfo.images):
-            if all(image.bpc == 1 for image in pageinfo.images):
-                device = 'pngmono'
-            elif all(image.bpc > 1 and image.color == Colorspace.index
-                     for image in pageinfo.images):
-                device = 'png256'
-            elif all(image.bpc > 1 and image.color == Colorspace.gray
-                     for image in pageinfo.images):
-                device = 'pnggray'
+    colorspaces = ['pngmono', 'pnggray', 'png256', 'png16m']
+    device_idx = 0
+    def at_least(cs):
+        return max(device_idx, colorspaces.index(cs))
+
+    for image in pageinfo.images:
+        if image.type_ != 'image':
+            continue  # ignore masks
+        if image.bpc > 1:
+            if image.color == Colorspace.index:
+                device_idx = at_least('png256')
+            elif image.color == Colorspace.gray:
+                device_idx = at_least('pnggray')
+            else:
+                device_idx = at_least('png16m')
+
+    device = colorspaces[device_idx]
 
     log.debug("Rasterize {0} with {1}".format(
               os.path.basename(input_file), device))
@@ -624,19 +645,18 @@ def select_visible_page_image(
         log.debug('{:4d}: JPEG input -> JPEG output'.format(
             page_number(image)))
         # If all images were JPEGs originally, produce a JPEG as output
-        im = Image.open(image)
+        with Image.open(image) as im:
+            # At this point the image should be a .png, but deskew, unpaper
+            # might have removed the DPI information. In this case, fall back to
+            # square DPI used to rasterize. When the preview image was
+            # rasterized, it was also converted to square resolution, which is
+            # what we want to give tesseract, so keep it square.
+            fallback_dpi = get_page_square_dpi(pageinfo, options)
+            dpi = im.info.get('dpi', (fallback_dpi, fallback_dpi))
 
-        # At this point the image should be a .png, but deskew, unpaper might
-        # have removed the DPI information. In this case, fall back to square
-        # DPI used to rasterize. When the preview image was rasterized, it
-        # was also converted to square resolution, which is what we want to
-        # give tesseract, so keep it square.
-        fallback_dpi = get_page_square_dpi(pageinfo, options)
-        dpi = im.info.get('dpi', (fallback_dpi, fallback_dpi))
-
-        # Pillow requires integer DPI
-        dpi = round(dpi[0]), round(dpi[1])
-        im.save(output_file, format='JPEG', dpi=dpi)
+            # Pillow requires integer DPI
+            dpi = round(dpi[0]), round(dpi[1])
+            im.save(output_file, format='JPEG', dpi=dpi)
     else:
         re_symlink(image, output_file, log)
 
@@ -658,25 +678,25 @@ def select_image_layer(
         log.debug("{:4d}: page eligible for lossless reconstruction".format(
             page_number(page_pdf)))
         re_symlink(page_pdf, output_file, log)
-    else:
-        pageinfo = get_pageinfo(image, context)
+        return
 
-        # We rasterize a square DPI version of each page because most image
-        # processing tools don't support rectangular DPI. Use the square DPI
-        # as it accurately describes the image. It would be possible to
-        # resample the image at this stage back to non-square DPI to more
-        # closely resemble the input, except that the hocr renderer does not
-        # understand non-square DPI. The sandwich renderer would be fine.
-        dpi = get_page_square_dpi(pageinfo, options)
-        layout_fun = img2pdf.get_fixed_dpi_layout_fun((dpi, dpi))
+    pageinfo = get_pageinfo(image, context)
 
-        with open(image, 'rb') as imfile, \
-                open(output_file, 'wb') as pdf:
-            log.debug('{:4d}: convert'.format(page_number(page_pdf)))
-            img2pdf.convert(
-                imfile, with_pdfrw=False,
-                layout_fun=layout_fun, outputstream=pdf)
-            log.debug('{:4d}: convert done'.format(page_number(page_pdf)))
+    # We rasterize a square DPI version of each page because most image
+    # processing tools don't support rectangular DPI. Use the square DPI as it
+    # accurately describes the image. It would be possible to resample the image
+    # at this stage back to non-square DPI to more closely resemble the input,
+    # except that the hocr renderer does not understand non-square DPI. The
+    # sandwich renderer would be fine.
+    dpi = get_page_square_dpi(pageinfo, options)
+    layout_fun = img2pdf.get_fixed_dpi_layout_fun((dpi, dpi))
+
+    with open(image, 'rb') as imfile, open(output_file, 'wb') as pdf:
+        log.debug('{:4d}: convert'.format(page_number(page_pdf)))
+        img2pdf.convert(
+            imfile, with_pdfrw=False,
+            layout_fun=layout_fun, outputstream=pdf)
+        log.debug('{:4d}: convert done'.format(page_number(page_pdf)))
 
 
 def render_hocr_page(
@@ -872,12 +892,8 @@ def get_pdfmark(base_pdf, options):
         except (KeyError, TypeError):
             return ''
 
-    pdfmark = {
-        '/Title': from_document_info('/Title'),
-        '/Author': from_document_info('/Author'),
-        '/Keywords': from_document_info('/Keywords'),
-        '/Subject': from_document_info('/Subject'),
-    }
+    pdfmark = {k: from_document_info(k) for k in
+        ('/Title', '/Author', '/Keywords', '/Subject', '/CreationDate')}
     if options.title:
         pdfmark['/Title'] = options.title
     if options.author:
@@ -898,6 +914,7 @@ def get_pdfmark(base_pdf, options):
         PROGRAM_NAME, VERSION,
         renderer_tag,
         tesseract.version())
+    pdfmark['/ModDate'] = encode_pdf_date(datetime.now(timezone.utc))
     return pdfmark
 
 
@@ -909,7 +926,20 @@ def generate_postscript_stub(
     options = context.get_options()
     pdf = pypdf.PdfFileReader(input_file)
     pdfmark = get_pdfmark(pdf, options)
-    generate_pdfa_ps(output_file, pdfmark)
+
+    ascii_docinfo = False
+    if ghostscript.version() >= '9.24':
+        ascii_docinfo = True
+        try:
+            for v in pdfmark.values():
+                v.encode('ascii', errors='strict')
+        except UnicodeEncodeError:
+            log.warning(
+                "Ghostscript 9.24+ does not support Unicode strings in "
+                " metadata. These will be converted to ASCII if possible."
+            )
+
+    generate_pdfa_ps(output_file, pdfmark, ascii_docinfo=ascii_docinfo)
 
 
 def skip_page(
@@ -967,6 +997,7 @@ def merge_pages_ghostscript(
     pdf_pages, _ = _merge_pages_common(
         input_files_groups, output_file, log, context)
     input_pdfinfo = context.get_pdfinfo()
+
     ghostscript.generate_pdfa(
         pdf_version=input_pdfinfo.min_version,
         pdf_pages=pdf_pages,
@@ -974,7 +1005,7 @@ def merge_pages_ghostscript(
         compression=options.pdfa_image_compression,
         log=log,
         threads=options.jobs or 1,
-        pdfa_part=('1' if options.output_type == 'pdfa-1' else '2'))
+        pdfa_part=options.output_type[-1])  # is pdfa-1, pdfa-2, or pdfa-3
     if fitz:
         doc = fitz.Document(output_file + '_toc.pdf')
         doc.setToC(input_pdfinfo.table_of_contents)
@@ -1031,7 +1062,7 @@ def merge_pages_mupdf(
     reader_metadata = pypdf.PdfFileReader(metadata_file)
     pdfmark = get_pdfmark(reader_metadata, options)
     pdfmark['/Producer'] = 'PyMuPDF ' + fitz.version[0]
-    pymupdf_metadata = {k[1:].lower() : v for k, v in pdfmark.items()}
+    pymupdf_metadata = {(k[1].lower() + k[2:]) : v for k, v in pdfmark.items()}
 
     for pdf_page in pdf_pages:
         page = fitz.open(pdf_page)
@@ -1065,7 +1096,7 @@ def merge_sidecars(
             if txt_file:
                 with open(txt_file, 'r', encoding="utf-8") as in_:
                     txt = in_.read()
-                    # Tesseract v4 alpha started adding form feeds in 
+                    # Tesseract v4 alpha started adding form feeds in
                     # commit aa6eb6b
                     # No obvious way to detect what binaries will do this, so
                     # for consistency just ignore its form feeds and insert our
@@ -1099,7 +1130,7 @@ def copy_final(
             sys.stdout.flush()
         else:
             # At this point we overwrite the output_file specified by the user
-            # use copyfileobj because then we use open() to create the file and 
+            # use copyfileobj because then we use open() to create the file and
             # get the appropriate umask, ownership, etc.
             with open(output_file, 'wb') as output_stream:
                 copyfileobj(input_stream, output_stream)
@@ -1274,7 +1305,7 @@ def build_pipeline(options, work_folder, log, context):
         output=os.path.join(work_folder, r'\1.rendered.pdf'),
         extras=[log, context])
     task_combine_layers.graphviz(fillcolor='"#00cc66"')
-    task_combine_layers.active_if(options.pdf_renderer == 'hocr' or 
+    task_combine_layers.active_if(options.pdf_renderer == 'hocr' or
                                   options.pdf_renderer == 'sandwich')
 
     # Tesseract OCR+PDF
