@@ -300,7 +300,8 @@ def test_autorotate_threshold(
 @pytest.mark.parametrize('renderer',RENDERERS)
 def test_ocr_timeout(renderer, resources, outpdf):
     out = check_ocrmypdf(resources / 'skew.pdf', outpdf,
-                         '--tesseract-timeout', '1.0')
+                         '--tesseract-timeout', '0.01',
+                         '--pdf-renderer', renderer)
     pdfinfo = PdfInfo(out)
     assert not pdfinfo[0].has_text
 
@@ -472,7 +473,7 @@ def test_tesseract_image_too_big(renderer, spoof_tesseract_big_image_error,
                                  resources, outpdf):
     check_ocrmypdf(
         resources / 'hugemono.pdf', outpdf, '-r',
-        '--pdf-renderer', renderer, 
+        '--pdf-renderer', renderer,
         '--max-image-mpixels', '0',
         env=spoof_tesseract_big_image_error)
 
@@ -607,7 +608,10 @@ def test_closed_streams(spoof_tesseract_noop, ocrmypdf_exec, resources, outpdf):
 
 
 def test_masks(spoof_tesseract_noop, resources, outpdf):
-    check_ocrmypdf(resources / 'masks.pdf', outpdf, env=spoof_tesseract_noop)
+    p, out, err = run_ocrmypdf(
+        resources / 'masks.pdf', outpdf, env=spoof_tesseract_noop)
+
+    assert p.returncode == ExitCode.ok
 
 
 def test_linearized_pdf_and_indirect_object(spoof_tesseract_noop,
@@ -715,7 +719,7 @@ def test_tesseract_config_notfound(renderer, resources, outdir):
         '--pdf-renderer', renderer,
         '--tesseract-config', cfg_file)
     assert "Can't open" in err, "No error message about missing config file"
-    assert p.returncode == ExitCode.ok
+    assert p.returncode == ExitCode.ok, err
 
 
 @pytest.mark.parametrize('renderer', RENDERERS)
@@ -825,6 +829,7 @@ def test_no_contents(spoof_tesseract_noop, resources, outpdf):
 @pytest.mark.parametrize('image', [
     'baiona.png',
     'baiona_gray.png',
+    'baiona_alpha.png',
     'congress.jpg'
     ])
 def test_compression_preserved(spoof_tesseract_noop, ocrmypdf_exec,
@@ -835,7 +840,6 @@ def test_compression_preserved(spoof_tesseract_noop, ocrmypdf_exec,
     output_file = str(outpdf)
 
     im = Image.open(input_file)
-
     # Runs: ocrmypdf - output.pdf < testfile
     with open(input_file, 'rb') as input_stream:
         p_args = ocrmypdf_exec + [
@@ -845,7 +849,12 @@ def test_compression_preserved(spoof_tesseract_noop, ocrmypdf_exec,
             stdin=input_stream, env=spoof_tesseract_noop)
         out, err = p.communicate()
 
-        assert p.returncode == ExitCode.ok
+        if im.mode in ('RGBA', 'LA'):
+            # If alpha image is input, expect an error
+            assert p.returncode != ExitCode.ok and b'alpha' in err
+            return
+
+        assert p.returncode == ExitCode.ok, err.decode('utf-8')
 
     pdfinfo = PdfInfo(output_file)
 
@@ -890,7 +899,7 @@ def test_compression_changed(spoof_tesseract_noop, ocrmypdf_exec,
             stdin=input_stream, env=spoof_tesseract_noop)
         out, err = p.communicate()
 
-        assert p.returncode == ExitCode.ok
+        assert p.returncode == ExitCode.ok, err
 
     pdfinfo = PdfInfo(output_file)
 
@@ -898,7 +907,7 @@ def test_compression_changed(spoof_tesseract_noop, ocrmypdf_exec,
 
     if compression == "jpeg":
         assert pdfimage.enc == Encoding.jpeg
-    elif compression == 'lossless':
+    else:
         assert pdfimage.enc not in (Encoding.jpeg, Encoding.jpeg2000)
 
     if im.mode.startswith('RGB') or im.mode.startswith('BGR'):
@@ -942,17 +951,22 @@ def test_sidecar_nonempty(spoof_tesseract_cache, resources, outpdf):
     assert 'the' in ocr_text
 
 
-def test_pdfa_1(spoof_tesseract_cache, resources, outpdf):
+@pytest.mark.parametrize('pdfa_level', ['1', '2', '3'])
+def test_pdfa_n(spoof_tesseract_cache, pdfa_level, resources, outpdf):
+    if pdfa_level == '3' and ghostscript.version() < '9.19':
+        pytest.xfail(reason='Ghostscript >= 9.19 required')
+
     check_ocrmypdf(
         resources / 'ccitt.pdf', outpdf,
-        '--output-type', 'pdfa-1',
+        '--output-type', 'pdfa-' + pdfa_level,
         env=spoof_tesseract_cache
     )
 
     pdfa_info = file_claims_pdfa(outpdf)
-    assert pdfa_info['conformance'] == 'PDF/A-1B'
+    assert pdfa_info['conformance'] == 'PDF/A-{}B'.format(pdfa_level)
 
 
+@pytest.mark.skipif(sys.version_info >= (3, 7, 0), reason='fixed')
 def test_bad_locale():
     env = os.environ.copy()
     env['LC_ALL'] = 'C'
@@ -980,7 +994,7 @@ def test_bad_utf8(spoof_tess_bad_utf8, renderer, resources, no_outpdf):
 
 
 @pytest.mark.skipif(
-    not tesseract.has_textonly_pdf(), 
+    not tesseract.has_textonly_pdf(),
     reason="issue only affects sandwich")
 def test_rotate_deskew_timeout(resources, outdir):
     check_ocrmypdf(
@@ -1003,7 +1017,7 @@ def test_rotate_deskew_timeout(resources, outdir):
 
 
 @pytest.mark.skipif(
-    PIL.PILLOW_VERSION < '5.0.0',
+    PIL.__version__ < '5.0.0',
     reason="Pillow < 5.0.0 doesn't raise the exception")
 def test_decompression_bomb(resources, outpdf):
     p, out, err = run_ocrmypdf(
@@ -1030,10 +1044,10 @@ def test_text_curves(spoof_tesseract_noop, resources, outpdf):
     check_ocrmypdf(
         resources / 'vector.pdf', outpdf, '--force-ocr',
         env=spoof_tesseract_noop)
-    
+
     info = PdfInfo(outpdf)
     assert len(info.pages[0].images) != 0, "force did not rasterize"
-    
+
 
 def test_dev_null(spoof_tesseract_noop, resources):
     p, out, err = run_ocrmypdf(
@@ -1076,4 +1090,4 @@ def test_skip_repair(spoof_tesseract_noop, resources, outpdf):
         outpdf,
         '--skip-repair',
         env=spoof_tesseract_noop
-    )
\ No newline at end of file
+    )
