@@ -18,10 +18,18 @@
 
 import pytest
 import PyPDF2 as pypdf
+import datetime
+from datetime import timezone
 
-from ocrmypdf.pdfa import file_claims_pdfa
+from ocrmypdf.pdfa import file_claims_pdfa, encode_pdf_date, decode_pdf_date
 from ocrmypdf.exceptions import ExitCode
 from ocrmypdf.lib import fitz
+from ocrmypdf.helpers import fspath
+from ocrmypdf.pdfa import (
+    file_claims_pdfa, encode_pdf_date, decode_pdf_date, generate_pdfa_ps,
+    SRGB_ICC_PROFILE
+)
+from ocrmypdf.exec import ghostscript
 
 # pytest.helpers is dynamic
 # pylint: disable=no-member
@@ -71,11 +79,19 @@ def test_override_metadata(spoof_tesseract_noop, output_type, resources,
 
     assert p.returncode == ExitCode.ok, err
 
-    reader = pypdf.PdfFileReader(outpdf)
+    before = pypdf.PdfFileReader(str(input_file))
+    after = pypdf.PdfFileReader(outpdf)
 
-    assert reader.documentInfo['/Title'] == german
-    assert reader.documentInfo['/Author'] == chinese
-    assert reader.documentInfo.get('/Keywords', '') == ''
+    if ghostscript.version() >= '9.24':
+        pytest.xfail('Ghostscript 9.24+ does not support Unicode DOCINFO')
+
+    assert after.documentInfo['/Title'] == german
+    assert after.documentInfo['/Author'] == chinese
+    assert after.documentInfo.get('/Keywords', '') == ''
+
+    before_date = decode_pdf_date(before.documentInfo['/CreationDate'])
+    after_date = decode_pdf_date(after.documentInfo['/CreationDate'])
+    assert before_date == after_date
 
     pdfa_info = file_claims_pdfa(outpdf)
     assert pdfa_info['output'] == output_type
@@ -115,3 +131,38 @@ def test_bookmarks_preserved(spoof_tesseract_noop, output_type, ocr_option,
     print(before_toc)
     print(after_toc)
     assert before_toc == after_toc
+
+
+def seconds_between_dates(date1, date2):
+    return (date2 - date1).total_seconds()
+
+
+@pytest.mark.parametrize('infile', ['trivial.pdf', 'jbig2.pdf'])
+@pytest.mark.parametrize('output_type', ['pdf', 'pdfa'])
+def test_creation_date_preserved(spoof_tesseract_noop, output_type, resources,
+                                 infile, outpdf):
+    input_file = resources / infile
+
+    before = pypdf.PdfFileReader(str(input_file)).getDocumentInfo()
+    check_ocrmypdf(
+        input_file, outpdf, '--output-type', output_type,
+        env=spoof_tesseract_noop)
+    after = pypdf.PdfFileReader(str(outpdf)).getDocumentInfo()
+
+    if not before:
+        # If there was input creation date, none should be output
+        # because of Ghostscript quirks we set it to null
+        # This test would be better if we had a test file with /DocumentInfo but
+        # no /CreationDate, which we don't
+        assert not after['/CreationDate'] or \
+                isinstance(after['/CreationDate'], pypdf.generic.NullObject)
+    else:
+        # We expect that the creation date stayed the same
+        date_before = decode_pdf_date(before['/CreationDate'])
+        date_after = decode_pdf_date(after['/CreationDate'])
+        assert seconds_between_dates(date_before, date_after) < 1000
+
+    # We expect that the modified date is quite recent
+    date_after = decode_pdf_date(after['/ModDate'])
+    assert seconds_between_dates(
+        date_after, datetime.datetime.now(timezone.utc)) < 1000
